#define _GNU_SOURCE
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>

#include "readelf.h"

/**
 * Nahraje do pameti staticky zkompilovanou binarku a spusti ji
 */
void load_and_start(FILE *f, Elf32_Ehdr *ehdr)
{
	for (int i = 0; i < ehdr->e_phnum; i++) {
		Elf32_Phdr phdr;
		readelf_program_header(f, i, ehdr, &phdr);

		// TODO: nastavit priznaky podle headeru
		mmap((void *) (phdr.p_vaddr), phdr.p_memsz, PROT_READ | PROT_WRITE | PROT_EXEC, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
		fseek(f, phdr.p_offset, SEEK_SET);
		fread((void *) phdr.p_vaddr, 1, phdr.p_memsz, f);
	}

	void (*start_fun)() = (void (*)()) ehdr->e_entry;
	start_fun();
}

/**
 * Zobrazi informace o spustitelnem ELF-souboru
 */
void list(FILE *f, Elf32_Ehdr *ehdr)
{
	printf("phnum:%i\n", ehdr->e_phnum);
	for (int i = 0; i < ehdr->e_phnum; i++) {
		Elf32_Phdr phdr;
		readelf_program_header(f, i, ehdr, &phdr);

		printf("%i: type: %x\tflags: %x\n", i, phdr.p_type, phdr.p_flags);
		if (phdr.p_type == PT_LOAD) {
			printf("file offs: %x\tmem addr:%x\n", phdr.p_offset, phdr.p_vaddr);
			printf("file size: %x\tmem size:%x\n", phdr.p_filesz, phdr.p_memsz);
		}
	}

}

int main(int argc, char **argv)
{
	Elf32_Ehdr ehdr;
	if (argc < 3) {
		printf("Usage: %s (list|run) file\n", argv[0]);
		return 0;
	}

	FILE *f = fopen(argv[2], "r");
	if (!f) {
		printf("Invalid file\n");
		return 1;
	}

	int status = readelf_file_header(f, &ehdr);
	if (status) {
		printf("Invalid file\n");
		return 1;
	}

	if (!strcmp(argv[1], "list")) list(f, &ehdr);
	else if (!strcmp(argv[1], "run")) load_and_start(f, &ehdr);
	else printf("Unknown command: %s\n", argv[1]);

	fclose(f);

	return 0;
}
